PI_deconvUknownth4het<-function(W,errortype,sigUj)

#Author: Aurore Delaigle
#compute 2-stage plug-in bandwidth for heteroscedastic kerndel deconvolution estimator as in:
#Delaigle, A. and Meister, A. (2008). Density estimation with heteroscedastic error. Bernoulli, 14, 562-579

#W: vector of contaminated data
#errortype: 'Lap' for the case where the error densities are all Laplace densities and 'norm' for the case where the error densities are all normal. 
#For other error distributions, simply redefine phiU below 
#sigUj: vector of length n which contains the parameters of each of the n Laplace or normal errors. Used only to define phiU.


# ------------------------------------------------------------------------------------------------------------------------------------------------------------------------
#								WARNINGS:
# ------------------------------------------------------------------------------------------------------------------------------------------------------------------------

# The range of t-values -1 and 1 correspond to the support of phiK.  
# If you change phiK and take a kernel for which phiK is not supported on [-1,1] you have to change -1 and 1 accordingly
#
# muK2 below is the second moment \int x^2 K(x) dx  of the kernel K defined below through phiK
# If you change phiK, you MUST change muK2 accordingly.
#
# ------------------------------------------------------------------------------------------------------------------------------------------------------------------------






{
# --------------------------------------------------------
# Preliminary calculations and initialisation of functions
# --------------------------------------------------------

#Sample size
n = length(W);


#Default values of phiU(t)=characteristic function of the errors
#If you want to consider another error type, simply replace phiU by the characteristic function of your error type

if (errortype=="Lap")
	{phiUk<-function(t,k) {return(1/(1+sigUj[k]^2*t^2));}
	varX=mean(W^2)-(mean(W))^2-2*sum(sigUj^2)/n;}
if (errortype=="norm")
	{phiUk<-function(t,k) {return(exp(-sigUj[k]^2*t^2/2));}
	varX=mean(W^2)-(mean(W))^2-sum(sigUj^2)/n;}

#phiK: Fourier transform of the kernel K. You can change this if you wish to use another kernel but make sure 
#you change the range of t-values, which should correspond to the support of phiK
phiK <-function(t) {return((1-t^2)^3);}


#second moment \int x^2 K(x) dx  of the kernel K
muK2 = 6;
RK=1024/3003/pi;

#Range of t-values (must correspond to the domain of phiK)
deltat = .0002;
t = seq(-1,1,deltat);
dim(t)=c(length(t),1);



#grid of h values where to search for a solution: you can change the default grid if no solution is found in this grid.
maxh=(max(W)-min(W))/10;

#NR bandwidth of the KDE estimator using the same kernel as we use in the DKDE case
hnaive=((8*sqrt(pi)*RK/3/muK2^2)^0.2)*sqrt(var(W))*n^(-1/5);
hgrid=seq(hnaive/3,maxh,(maxh-hnaive/3)/100);
lh = length(hgrid);
dim(hgrid)=c(1,lh);


#Estimator of the standard deviation of X
stdevx = max(sqrt(varX),1/n);



#Quantities that will be needed several times in the computations below
toverh=t%*%(1/hgrid);
phiK2=(phiK(t))^2;
matphiU=matrix(0,length(t),length(hgrid));
phiU2=matphiU;
for (k in 1:n)
	{matphiU=phiUk(toverh,k);
	phiU2=phiU2+matphiU^2;}




# --------------------------------------------
# Estimate theta4 by normal reference method     
# --------------------------------------------

th4 = stdevx^(-9)*105/(32*sqrt(pi)); 

# ------------------------------------------------------
# Find bandwidth h3 for computing th3, then compute th3
# ------------------------------------------------------

rr=3;
term1= -hgrid^2*muK2*th4;
term2=kronecker(matrix(1,1,lh),t^(2*rr)*phiK2)/phiU2;
term2=apply(term2,2,sum)*deltat;
term2=term2/(2*pi*hgrid^(2*rr+1));

ABias2 = (term1 + term2)^2;

#Print the index of the minimiser of Abias2 to see if we are inside the grid (if not, enlarge the grid of bandwidths)
indh3=which.min(ABias2)
h3 = hgrid[indh3];

#Estimate empirical characteristic function of W
OO=outerop(t/h3,W,"*");

# Compute phiU(-t/h) -- since phiU is symmetric, this is the same as phiU(t/h)
matphiU=OO;
for (k in 1:n)
	{matphiU[,k]=phiUk(t/h3,k);}


phiUth=apply(matphiU^2,1,sum);

#Estimate empirical characteristic function of W.
rehatphiX=apply(cos(OO)*matphiU,1,sum)/phiUth;
imhatphiX=apply(sin(OO)*matphiU,1,sum)/phiUth;

rm(OO);

#Compute th3
normhatphiX2=rehatphiX^2+imhatphiX^2;
th3 = sum(t^(2*rr) * normhatphiX2 * phiK2);
th3 = th3*deltat/(2*pi*h3^(2*rr+1));

# -----------------------------------------------------
# Find bandwidth h2 for computing th2, then compute th2
# -----------------------------------------------------


rr=2;
term1= -hgrid^2*muK2*th3;
term2=kronecker(matrix(1,1,lh),t^(2*rr)*phiK2)/phiU2;
term2=apply(term2,2,sum)*deltat/(2*pi*hgrid^(2*rr+1));
ABias2 = (term1 + term2)^2;

#Print the index of the minimiser of Abias2 to see if we are inside the grid (if not, enlarge the grid of bandwidths)
indh2=which.min(ABias2)
h2 = hgrid[indh2];

#Estimate empirical characteristic function of W
OO=outerop(t/h2,W,"*");
# Compute phiU(-t/h) -- since phiU is symmetric, this is the same as phiU(t/h)
matphiU=OO;
for (k in 1:n)
	{matphiU[,k]=phiUk(t/h2,k);}

phiUth=apply(matphiU^2,1,sum);

#Estimate empirical characteristic function of W.
rehatphiX=apply(cos(OO)*matphiU,1,sum)/phiUth;
imhatphiX=apply(sin(OO)*matphiU,1,sum)/phiUth;

rm(OO);


#Compute th2
normhatphiX2=rehatphiX^2+imhatphiX^2;
th2 = sum(t^(2*rr) * normhatphiX2 * phiK2);
th2=th2*deltat/(2*pi*h2^(2*rr+1));


# ------------------------------------------------------------------------------------------------------
# Finally, compute the bandwidth that minimises the AMISE of the deconvolution kernel density estimator
# ------------------------------------------------------------------------------------------------------

term1=hgrid^4*muK2^2*th2/4;
term2=kronecker(matrix(1,1,lh),phiK2)/phiU2;
term2=apply(term2,2,sum)*deltat/(2*pi*hgrid);
AMISE=term1+term2;

indh=which.min(AMISE)
hPI = hgrid[indh];

return (hPI)}

